# -*- coding: utf-8 -*-

from __future__ import unicode_literals

'''
Tool-box for efficient build and debug in Python. 
Espacially for Scientific Computing and Computer Vision.
'''
__version__ = "0.10.13"
__short_description__ = "Tool-box for efficient build and debug in Python. Especially for Scientific Computing and Computer Vision."
__license__ = "MIT"
__author__ = "DIYer22"
__author_email__ = "ylxx@live.com"
__maintainer__ = "DIYer22"
__maintainer_email__ = "ylxx@live.com"
__github_username__ = "DIYer22"
__github_url__ = "https://github.com/DIYer22/boxx"
__support__ = "https://github.com/DIYer22/boxx/issues"

from . import ylsys
from . import ylcompat
from . import tool
from . import ylimg
from . import ylsci
from . import ylml
from . import yldb
from . import undetermined



from .tool import *
from .ylsys import *
from .ylcompat import *
from .ylimg import *
from .ylml import *
from .ylsci import *
from .yldb import *

if __name__ == '__main__':
    pass

